package com.yunzhi.niucai.controller;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipayOpenAuthTokenAppRequest;
import com.alipay.api.response.AlipayOpenAuthTokenAppResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaAlipay;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShopApp;
import com.yunzhi.niucai.service.ITaShopAppService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaAlipayService;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-27
 */

@Api(tags = "")
@RestController
@RequestMapping("/aliPay")
public class TaAlipayController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaAlipayController.class);


    @Autowired
    public ITaShopAppService iTaShopAppService;

    @Autowired
    private ITaAlipayService aliPayService;

    /**
     * 手机app获取唤醒支付宝的参数
     * @param request
     * @param taOrder
     * @return
     * @throws Exception
     */
    @PostMapping("/create")
    @ApiOperation(value = "创建订单", notes = "创建订单", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean create(HttpServletRequest request, @RequestBody TaOrder taOrder) throws Exception {
        //检查token的使用时间
        TaShopApp taShopApp = iTaShopAppService.getOne(new QueryWrapper<TaShopApp>().eq("shop_id", taOrder.getShopId()));
        checkAliPayToken(taShopApp);
        return aliPayService.orderPaying(taOrder,taShopApp);
    }

    /**
     * 检查alipay的token是否24小时未使用
     * @param taShopApp
     * @return
     */
    private void checkAliPayToken(TaShopApp taShopApp) throws AlipayApiException {
        if (taShopApp.getUpdateTokenDate().plusDays(1).isBefore(LocalDateTime.now())){
            return;
        }
        TaAlipay taAlipay = aliPayService.getById(taShopApp.getAlipayId());
        getAliPayToken(taAlipay,"refresh_token",taShopApp.getAppRefreshToken());
    }

    /**
     * 获取或刷新支付宝token
     * @param taAlipay
     * @param grantType
     * @param commonToken
     * @throws AlipayApiException
     */
    private void getAliPayToken(TaAlipay taAlipay,String grantType,String commonToken) throws AlipayApiException {
        AlipayClient alipayClient = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do", taAlipay.getAppid(), taAlipay.getMerchantPrivateKey(), "json", "GBK", taAlipay.getAlipayPublicKey(), "RSA2");
        AlipayOpenAuthTokenAppRequest request = new AlipayOpenAuthTokenAppRequest();
        if (grantType.equals("refresh_token")){
            request.setBizContent("{" +
                    "\"grant_type\":\"refresh_token\"," +
                    "\"refresh_token\":\""+commonToken+"\"," +
                    "  }");
        }else if (grantType.equals("authorization_code")){
            request.setBizContent("{" +
                    "\"grant_type\":\"authorization_code\"," +
                    "\"code\":\""+commonToken+"\"," +
                    "  }");
        }

        AlipayOpenAuthTokenAppResponse response = alipayClient.execute(request);
        if(response.isSuccess()){
            System.out.println("调用成功");
            TaShopApp taShopApp = iTaShopAppService.getOne(new QueryWrapper<TaShopApp>().eq("appid", response.getAuthAppId()));
            taShopApp.setAppAuthToken(response.getAppAuthToken());
            taShopApp.setAppRefreshToken(response.getAppRefreshToken());
            taShopApp.setUpdateTokenDate(LocalDateTime.now());
            taShopApp.setAlipayId(taAlipay.getAlipayId());
            iTaShopAppService.updateById(taShopApp);
        } else {
            System.out.println("调用失败");
        }
    }

    /**
     * 支付成功回调
     * @param request
     */
    @GetMapping("/callback")
    public void callback(HttpServletRequest request){
        String tradeNo = request.getParameter("trade_no");
        String appId = request.getParameter("app_id");
        String outTradeNo = request.getParameter("out_trade_no");
        String tradeStatus = request.getParameter("trade_status");
        aliPayService.payCallback(tradeNo,outTradeNo,tradeStatus);

    }

    /**
     * 应用授权成功回调
     * @param appId
     * @param appAuthCode
     * @throws AlipayApiException
     */
    @GetMapping("/auth/notice")
    public void aliPayAuthNotice(@RequestParam("app_id") String appId,@RequestParam("app_auth_code") String appAuthCode) throws AlipayApiException {
        TaAlipay taAlipay =aliPayService.getOne(new QueryWrapper<TaAlipay>().eq("appid",appId));
        getAliPayToken(taAlipay,"authorization_code",appAuthCode);
    }
}
